const Redis = require("ioredis");

class Cache {
	constructor(options) {
		this.client = new Redis({
			host: options.host,
			port: options.port,
		});

		this.client.on("error", (err) => console.log(`Redis connection error: ${err}`));
		this.client.on("ready", () => console.log("Redis connection is ready!"));
	}

	async invalidate(key) {
		if (this.client.status !== "ready") {
			console.log("Reddis connection error!");
			return;
		}

		try {
			await this.client.del(key);
		} catch (err) {
			console.log(err);
		}
	}

	async setValue(key, value, expSeconds) {
		if (this.client.status !== "ready") {
			console.log("Reddis connection error!");
			return;
		}

		if (expSeconds) {
			await this.client.setex(key, expSeconds, typeof (value) === "object" ? JSON.stringify(value) : value);
		} else {
			await this.client.set(key, typeof (value) === "object" ? JSON.stringify(value) : value);
		}
	}

	async getValue(key) {
		const result = this.client.status === "ready" ? await this.client.get(key) : null;
		return result;
	}

	async getKeys(pattern) {
		const result = this.client.status === "ready" ? await this.client.keys(pattern) : null;
		return result;
	}
}

module.exports = Cache;
