/* eslint-disable no-unused-expressions */
/* eslint-disable consistent-return */
/* eslint-disable no-underscore-dangle */
const { Types } = require("mongoose");

const model = (connection) => (name, schema, collection = "") => {
	const collectionName = collection || (!name.endsWith("s") ? `${name}s` : name);
	return connection.model(name, schema, collectionName);
};

const convertToMultiple = (convertor) => (entities) => {
	if (Array.isArray(entities)) {
		return entities.map((entity) => convertor(entity));
	}
	return [convertor(entities)];
};

const conditionHasId = (condition) => Object.prototype.hasOwnProperty.call(condition, "_id");

const idIsValid = (id) => Types.ObjectId.isValid(id);

const pureFetch = (mongomodel, domainconvertor = (entity) => entity) => async (condition = {}, paging = null, includedFields = null) => {
	if (conditionHasId(condition) && !idIsValid(condition._id)) {
		return [];
	}

	const doc = await mongomodel.find(condition, includedFields, paging);
	if (doc.length === 0) {
		return [];
	}

	console.log(doc);

	const domainConvertors = convertToMultiple(domainconvertor);
	return domainConvertors(doc);
};

const save = (mongomodel, modelconvertor = (entity) => entity, selector) => async (entities) => {
	const modelConvertors = convertToMultiple(modelconvertor);

	const models = modelConvertors(entities);
	if (!models || models.length === 0) {
		return;
	}

	if (models.length > 1) {
		const bulk = mongomodel.collection.initializeOrderedBulkOp();

		models.map((m) => bulk.find(selector(m)).upsert().updateOne(m));

		const result = await bulk.execute();

		return result;
	}

	const dataModel = models[0];

	if (selector) {
		const original = await pureFetch(mongomodel)(selector(dataModel));
		if (!original || original.length === 0) {
			const result = await dataModel.save();
			return result;
		}

		const storedCreatedAt = new Date(original[0].createdAt);
		const storedUpdatedAt = new Date(original[0].updatedAt);
		// eslint-disable-next-line no-underscore-dangle
		const storedId = original[0]._doc._id;

		// eslint-disable-next-line no-underscore-dangle
		dataModel._doc._id = storedId;
		dataModel.isNew = false;
		dataModel.createdAt = storedCreatedAt;
		dataModel.updatedAt = storedUpdatedAt;

		const result = await dataModel.save();
		return result;
	}

	const result = dataModel.save();
	return result;
};

const fetch = (mongomodel, domainconvertor = (entity) => entity) => async (condition = Object()) => pureFetch(mongomodel, domainconvertor)(condition);

const fetchById = (mongomodel, domainconvertor = (entity) => entity) => async (id) => pureFetch(mongomodel, domainconvertor)({ _id: id });

const remove = (mongomodel) => async (condition = Object()) => (await mongomodel.deleteMany(condition)).n > 0;

const copyTo = (mongomodel) => async (to) => {
	await mongomodel.aggregate([{ $match: {} }, { $out: to }]);
};

const convertIdInQuery = (query) => {
	const modifiedQuery = query;

	if (query && (query.Id || query.id)) {
		const id = query.Id || query.id;
		// eslint-disable-next-line no-underscore-dangle
		modifiedQuery._id = id;
		query.Id && delete modifiedQuery.Id;
		query.id && delete modifiedQuery.id;
	}

	return modifiedQuery;
};

const geoNear = (mongomodel, domainconvertor = (entity) => entity) => async (lat, lon, condition = {}, maxDistance = 50000, limit = 20) => {
	const result = await mongomodel.aggregate([
		{
			$geoNear: {
				near: { type: "Point", coordinates: [lon, lat] },
				spherical: true,
				distanceField: "calcDistance",
				maxDistance,
				query: condition,
			},
		},
		{
			$limit: limit,
		},
	]);

	const domainConvertors = convertToMultiple(domainconvertor);
	return domainConvertors(result);
};

module.exports = {
	model,
	save,
	fetch,
	fetchById,
	remove,
	copyTo,
	convertToMultiple,
	convertIdInQuery,
	geoNear,
};
