const { Pool } = require("pg");
const { convertToMultiple } = require("./rdshelper");

const createPool = (user, host, database, password, port) => new Pool({
	user: user || process.env.PGUSER,
	host: host || process.env.PGHOST,
	database: database || process.env.PGDATABASE,
	password: password || process.env.PGPASSWORD,
	port: port || process.env.PGPORT,
});

const query = (pool, domainConvertor) => async (q, args = []) => {
	const client = await pool.connect();
	let response;
	try {
		const res = await client.query(q, args);
		response = res.rows;
		if (domainConvertor) {
			const convertors = convertToMultiple(domainConvertor);
			response = convertors(response);
		}
	} finally {
		client.release();
	}

	return response;
};

module.exports = { createPool, query };
