const ScooterService = require("../services/scooter");
const ScooterDataAccess = require("../data/models/scooter");
const required = require("../decorators/argumentRequired");
const ErrorResult = require("../objects/errorResult");
const ErrorCodes = require("../objects/errorCodes");

const dataAccess = new ScooterDataAccess();
const service = new ScooterService(dataAccess);

const scooter = async (app) => {
	app.post("/save", async (req) => {
		required("code")(req.body);

		const response = await service.save(req.body);
		return response;
	});

	app.post("/listAvailables", async (req) => {
		required("lat", "lon", "vehicleType")(req.body);

		const {
			lat, lon, maxDistance, limit, vehicleType,
		} = req.body;

		if (!Array.isArray(vehicleType)) {
			return ErrorResult(ErrorCodes.MISSING_PARAMETERS);
		}

		const response = await service.listAvailables(lat, lon, vehicleType, maxDistance, limit);
		return response;
	});
};

module.exports = scooter;
